// -*- C++ -*-
/*!
 * @file  USBCameraAcquire.cpp
 * @brief USB Camera Acquire component
 * @date $Date$
 *
 * $Id$
 */

#include "USBCameraAcquire.h"
#include <iostream>
using namespace std;

// Module specification
// <rtc-template block="module_spec">
static const char* usbcameraacquire_spec[] =
  {
    "implementation_id", "USBCameraAcquire",
    "type_name",         "USBCameraAcquire",
    "description",       "USB Camera Acquire component",
    "version",           "1.0.0",
    "vendor",            "AIST",
    "category",          "example",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1000.0",
    "conf.default.device_num",  "0",
    ""
  };
// </rtc-template>
  int device_num_old = 1000;
  bool imgflg = false;
/*!
 * @brief constructor
 * @param manager Maneger Object
 */
USBCameraAcquire::USBCameraAcquire(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_outOut("out", m_out),

    // </rtc-template>
    dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  registerOutPort("out", m_outOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
USBCameraAcquire::~USBCameraAcquire()
{
}



RTC::ReturnCode_t USBCameraAcquire::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("device_num", m_device_num, "0");

  // </rtc-template>
  return RTC::RTC_OK;
}



RTC::ReturnCode_t USBCameraAcquire::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t USBCameraAcquire::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCameraAcquire::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t USBCameraAcquire::onActivated(RTC::UniqueId ec_id)
{
    //JfoCX̒T
/*
    if(NULL==(m_capture = cvCaptureFromCAM(CV_CAP_ANY))){
        cout<<"No Camera Device"<<endl;
        return RTC::RTC_ERROR;
    }

    return RTC::RTC_OK;
*/
    return RTC::RTC_OK;
}


RTC::ReturnCode_t USBCameraAcquire::onDeactivated(RTC::UniqueId ec_id)
{
  //Jp̉
  if(m_capture != NULL)
  {
	cvReleaseCapture(&m_capture);
  }

  device_num_old = 1000;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t USBCameraAcquire::onExecute(RTC::UniqueId ec_id)
{
    static coil::TimeValue tm_pre;
    static int count = 0;
    IplImage *cam_frame = NULL;
    
    if(m_device_num != device_num_old){
    
		if (device_num_old != 1000)
		{
			cvReleaseCapture(&m_capture);
		}

		device_num_old = m_device_num;

		if(NULL==(m_capture = cvCaptureFromCAM(device_num_old))){
			cout<<"No Camera Device"<<endl;
			imgflg = false;
			//return RTC::RTC_ERROR;
		}else{
			imgflg = true;	
		}
    }

	if(imgflg == true){
	    cam_frame = cvQueryFrame(m_capture);
	    if(NULL == cam_frame)
	    {
			std::cout << "Bad frame or no frame!!" << std::endl;
			return RTC::RTC_ERROR;
	    }

	    IplImage* frame = cvCreateImage(cvGetSize(cam_frame), 8, 3);

	    if(cam_frame ->origin == IPL_ORIGIN_TL)
	    cvCopy(cam_frame, frame);
	    else
	    cvFlip(cam_frame, frame);

	    int len = frame->nChannels * frame->width * frame->height;

	    // ʂ̃TCY
	    m_out.pixels.length(len);
	    m_out.width  = frame->width;
	    m_out.height = frame->height;

	    memcpy((void *)&(m_out.pixels[0]), frame->imageData,len);

	    //摜\    
	    //cvShowImage("CaptureImage", frame);
	    cvWaitKey(1);

	    cvReleaseImage(&frame);

	    // qĂR|[lgł܂Ɩ蔭
	    m_outOut.write();

	    if (count > 100)
	    {
		count = 0;
		coil::TimeValue tm;
		tm = coil::gettimeofday();

		double sec(tm - tm_pre);
		
		if (sec > 1.0 && sec < 1000.0)
		{
		    std::cout << 100/sec << " [FPS]" << std::endl;
		}

		tm_pre = tm;
	    }
	    ++count;
    }
	

    return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t USBCameraAcquire::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCameraAcquire::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCameraAcquire::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCameraAcquire::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t USBCameraAcquire::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void USBCameraAcquireInit(RTC::Manager* manager)
  {
    coil::Properties profile(usbcameraacquire_spec);
    manager->registerFactory(profile,
                             RTC::Create<USBCameraAcquire>,
                             RTC::Delete<USBCameraAcquire>);
  }
  
};
