// -*- C++ -*-
/*!
 * @file  ObjectTracking.cpp
 * @brief Objecttrack component
 * @date $Date$
 *
 * $Id$
 */

#include "ObjectTracking.h"

// Module specification
// <rtc-template block="module_spec">
static const char* objecttracking_spec[] =
  {
    "implementation_id", "ObjectTracking",
    "type_name",         "ObjectTracking",
    "description",       "Objecttrack component",
    "version",           "1.0.0",
    "vendor",            "AIST",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1000.0",
    // Configuration variables
    "conf.default.image_height", "240",
    "conf.default.image_width", "320",
    ""
  };
// </rtc-template>

IplImage	*inputImage = NULL;			//͂ꂽIplImage
IplImage	*resultImage = NULL;			//	ʕ\pIplImage
IplImage	*hsvImage = NULL;			//	HSV\FnpIplImage
IplImage	*hueImage = NULL;			//	HSV\FnH`lpIplImage
IplImage	*maskImage = NULL;			//	}XN摜pIplImage
IplImage	*backprojectImage = NULL;	//	obNvWFNV摜pIplImage
IplImage	*histImage = NULL;			//	qXgO`pIplImage
IplImage	*grayImage = NULL;			//	O[XP[摜pIplImage

CvHistogram	*hist = NULL;				//	qXgOp\

IplImage	*frameImage;	//	Lv`摜pIplImage
CvCapture	*capture;		//	L[͌ʂi[ϐ
int count  = 0;
int g_temp_w = 0;
int g_temp_h = 0;

//	[hIptO
int	backprojectMode = HIDDEN_BACKPROJECTION;
int	selectObject = SELECT_OFF;
int	trackObject = TRACKING_STOP;
int showHist = SHOW_HISTOGRAM;

//	CamShiftgbLOpϐ
CvPoint			origin;
CvRect			selection;
CvRect			trackWindow;
CvBox2D			trackRegion;
CvConnectedComp	trackComp;

//	qXgOpϐ
int		hdims = H_DIMENSION;		//	qXgO̎
float	hRangesArray[] = {H_RANGE_MIN, H_RANGE_MAX};	//qXgÕW
float	*hRanges = hRangesArray;
int		vmin = V_MIN;
int		vmax = V_MAX;

//char	*windowNameObjectTracking = "CaptureImage";

//
//	}EXhbOɂďǐ՗̈w肷
//
//	:
//		event	: }EX{^̏
//		x		: }EX݃|CgĂxW
//		y		: }EX݃|CgĂyW
//		flags	: {vOł͖gp
//		param	: {vOł͖gp
//
static void on_mouse( int event, int x, int y, int flags, void* param ){
	//	摜擾ĂȂ΁AsȂ
	if( resultImage == NULL ){
        return;
	}

	//	_̈ʒuɉy̒l𔽓]i摜̔]ł͂Ȃj
	if( resultImage->origin == 1 ){
        y = resultImage->height - y;
	}
	//	}EX̍{^ĂΈȉ̏s
    if( selectObject == SELECT_ON ){
        selection.x = MIN( x, origin.x );
        selection.y = MIN( y, origin.y );
        selection.width = selection.x + CV_IABS( x - origin.x );
        selection.height = selection.y + CV_IABS( y - origin.y );
        
        selection.x = MAX( selection.x, 0 );
        selection.y = MAX( selection.y, 0 );
        selection.width = MIN( selection.width, resultImage->width );
        selection.height = MIN( selection.height, resultImage->height );
        selection.width = selection.width - selection.x;
        selection.height = selection.height - selection.y;
    }
	//	}EX̍{^̏Ԃɂď𕪊
    switch( event ){
		case CV_EVENT_LBUTTONDOWN:
			//	}EX̍{^ꂽ̂ł΁A
			//	_ёIꂽ̈ݒ
			origin = cvPoint( x, y );
			selection = cvRect( x, y, 0, 0 );
			selectObject = SELECT_ON;
			break;
		case CV_EVENT_LBUTTONUP:
			//	}EX̍{^ꂽƂAwidthheightǂł΁A
			//	trackObjecttOTRACKING_STARTɂ
			selectObject = SELECT_OFF;
			if( selection.width > 0 && selection.height > 0 ){
				trackObject = TRACKING_START;
			}
			break;
    }
}

//
//	͂ꂽ1̐FlRGBɕϊ
//
//	:
//		hue		: HSV\FnɂFlH
//	߂lF
//		CvScalar: RGB̐FBGȐŊi[ꂽRei
//
CvScalar hsv2rgb( float hue ){
	IplImage *rgbValue, *hsvValue;
	rgbValue = cvCreateImage( cvSize(1,1), IPL_DEPTH_8U, 3 );
	hsvValue = cvCreateImage( cvSize(1,1), IPL_DEPTH_8U, 3 );

	hsvValue->imageData[0] = hue;	//	FlH
	hsvValue->imageData[1] = 255;	//	ʓxlS
	hsvValue->imageData[2] = 255;	//	xlV
	
	//	HSV\FnRGB\Fnɕϊ
	cvCvtColor( hsvValue, rgbValue, CV_HSV2BGR );

	return cvScalar(	(unsigned char)rgbValue->imageData[0], 
						(unsigned char)rgbValue->imageData[1], 
						(unsigned char)rgbValue->imageData[2], 
						0 );

	//	
	cvReleaseImage( &rgbValue );
	cvReleaseImage( &hsvValue );
}


//
//	}EXIꂽǐ՗̈ɂHSVHlŃqXgO쐬AqXgO̕`܂łs
//
//	:
//		hist		: mainŐ錾ꂽqXgOp\
//		hsvImage	: ͉摜HSV\FnɕϊꂽIplImage
//		maskImage	: }XN摜pIplImage
//		selection	: }EXőIꂽ`̈
//
void CalculateHist( CvHistogram	*hist, IplImage *hsvImage, IplImage *maskImage, CvRect selection ){
	int		i;
	int		binW;	//	qXgO̊eŕA摜ł̕
	int		val;	//	qXgO̕px
	float	maxVal;	//	qXgO̍őpx


	//	hsv摜̊efl͈͓̔ɓĂ邩`FbNA
	//	}XN摜maskImage쐬
	cvInRangeS( hsvImage, 
				cvScalar( H_RANGE_MIN, S_MIN, MIN(V_MIN,V_MAX), 0 ),
				cvScalar( H_RANGE_MAX, S_MAX, MAX(V_MIN,V_MAX), 0 ), 
				maskImage );
	//	hsvImagêAƂɕKvH`lhueImageƂĕ
	cvSplit( hsvImage, hueImage, 0, 0, 0 );
	//	trackObjectTRACKING_STARTԂȂAȉ̏s
	if( trackObject == TRACKING_START ){
		//	ǐ՗̈̃qXgOvZhistImageւ̕`
		maxVal = 0.0;

		cvSetImageROI( hueImage, selection );
        cvSetImageROI( maskImage, selection );
        //	qXgOvZAől߂
		cvCalcHist( &hueImage, hist, 0, maskImage );
		cvGetMinMaxHistValue( hist, 0, &maxVal, 0, 0 );
        //	qXgȌcipxj0-255̃_Ci~bNWɐK
		if( maxVal == 0.0 ){
			cvConvertScale( hist->bins, hist->bins, 0.0, 0 );
		} else{
			cvConvertScale( hist->bins, hist->bins, 255.0 / maxVal, 0 );
		}
		//	hue,mask摜ɐݒ肳ꂽROIZbg
		cvResetImageROI( hueImage );
        cvResetImageROI( maskImage );

        trackWindow = selection;
        //	trackObjectTRACKING_NOWɂ
		trackObject = TRACKING_NOW;

		//	qXgO摜[NA
        cvSetZero( histImage );
		//	er̕߂
        binW = histImage->width / hdims;
		//	qXgO`悷
        for( i = 0; i < hdims; i++ ){
			val = cvRound( cvGetReal1D(hist->bins,i) * histImage->height / 255 );
            CvScalar color = hsv2rgb( i * 180.0 / hdims );
            cvRectangle(	histImage, 
							cvPoint( i * binW, histImage->height ), 
							cvPoint( (i+1) * binW, histImage->height - val ),
							color,
							-1, 
							8, 
							0	);
		}
	}
}

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ObjectTracking::ObjectTracking(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_orig_imgIn("img_original", m_orig_img),
	m_eventIn("m_event", m_event),
	m_xIn("m_x", m_x),
	m_yIn("m_y", m_y),
    m_out_imgOut("img_output", m_out_img),
    m_hist_imgOut("img_hist", m_hist_img)

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("img_original", m_orig_imgIn);
  registerInPort("m_event", m_eventIn);
  registerInPort("m_x", m_xIn);
  registerInPort("m_y", m_yIn);
  // Set OutPort buffer
  registerOutPort("img_output", m_out_imgOut);
  registerOutPort("img_hist", m_hist_imgOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
ObjectTracking::~ObjectTracking()
{
}



RTC::ReturnCode_t ObjectTracking::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("image_height", m_img_height, "240");
  bindParameter("image_width", m_img_width, "320");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ObjectTracking::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectTracking::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectTracking::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ObjectTracking::onActivated(RTC::UniqueId ec_id)
{	
	g_temp_w = 0;
	g_temp_h = 0;
	
	//em
	inputImage = NULL;
	resultImage = NULL;
	hsvImage = NULL;
	hueImage = NULL;
	maskImage = NULL;
	backprojectImage = NULL;
	grayImage = NULL;
	histImage = NULL;

	return RTC::RTC_OK;
}


RTC::ReturnCode_t ObjectTracking::onDeactivated(RTC::UniqueId ec_id)
{
	//
	if(inputImage != NULL){
		cvReleaseImage(&inputImage);
	}
	if(resultImage != NULL){
		cvReleaseImage(&resultImage);
	}
	if(hsvImage != NULL){
		cvReleaseImage(&hsvImage);
	}
	if(hueImage != NULL){
		cvReleaseImage(&hueImage);
	}
	if(hueImage != NULL){
		cvReleaseImage(&maskImage);
	}
	if(backprojectImage != NULL){
		cvReleaseImage(&backprojectImage);
	}
	if(grayImage != NULL){
		cvReleaseImage(&grayImage);
	}
	if(histImage != NULL){
		cvReleaseImage(&histImage);
	}

	return RTC::RTC_OK;
}


RTC::ReturnCode_t ObjectTracking::onExecute(RTC::UniqueId ec_id)
{
	int		key;	//	L[͌ʂi[ϐ
	int		i;
	int		j;
	
	int x;
	int y;
	int mouse_event;

	//	Snakep̃p[^
	float alpha = 1.0;		//	AGlM[̏d݃p[^
	float beta = 0.5;		//	ȗ̏d݃p[^
	float gamma = 1.5;		//	摜GlM[̏d݃p[^
	CvPoint pt[SEGMENT];	//	_̍W
	CvSize window;			//	ŏlTߖTTCY
	window.width = WINDOW_WIDTH;	
	window.height = WINDOW_HEIGHT;
	CvTermCriteria crit;
	crit.type = CV_TERMCRIT_ITER;		//	I̐ݒ
	crit.max_iter = ITERATION_SNAKE;	//	֐̍ő唽

	if(m_orig_imgIn.isNew()){

		m_orig_imgIn.read();
			
		//em
		if(inputImage == NULL){
			inputImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 3);
		}
		if(g_temp_w != m_orig_img.width || g_temp_h != m_orig_img.height){
			cvReleaseImage(&inputImage);
			inputImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 3);
		}
		if(resultImage == NULL){
			resultImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 3);
		}
		if(g_temp_w != m_orig_img.width || g_temp_h != m_orig_img.height){
			cvReleaseImage(&resultImage);
			resultImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 3);
		}
		resultImage->origin = inputImage->origin;
		if(hsvImage == NULL){
			hsvImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 3);
		}
		if(g_temp_w != m_orig_img.width || g_temp_h != m_orig_img.height){
			cvReleaseImage(&hsvImage);
			hsvImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 3);
		}
		if(hueImage == NULL){
			hueImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 1);
		}
		if(g_temp_w != m_orig_img.width || g_temp_h != m_orig_img.height){
			cvReleaseImage(&hueImage);
			hueImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 1);
		}
		if(maskImage == NULL){
			maskImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 1);
		}
		if(g_temp_w != m_orig_img.width || g_temp_h != m_orig_img.height){
			cvReleaseImage(&maskImage);
			maskImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 1);
		}
		if(backprojectImage == NULL){
			backprojectImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 1);
		}
		if(g_temp_w != m_orig_img.width || g_temp_h != m_orig_img.height){
			cvReleaseImage(&backprojectImage);
			backprojectImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 1);
		}
		if(grayImage == NULL){
			grayImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 1);
		}
		if(g_temp_w != m_orig_img.width || g_temp_h != m_orig_img.height){
			cvReleaseImage(&grayImage);
			grayImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 1);
		}
		//	qXgO\̂̎gp錾
		if(hist == NULL){
			hist = cvCreateHist( 1, &hdims, CV_HIST_ARRAY, &hRanges, 1 );
		}
		//	qXgOp̉摜mۂA[NA
		if(histImage == NULL){
			histImage = cvCreateImage( cvSize(HISTIMAGE_WIDTH, HISTIMAGE_HEIGHT), IPL_DEPTH_8U, 3 );
		}
		if(g_temp_w != m_orig_img.width || g_temp_h != m_orig_img.height){
			cvReleaseImage(&histImage);
			histImage = cvCreateImage(cvSize(m_orig_img.width, m_orig_img.height), IPL_DEPTH_8U, 3);
		}
		cvSetZero( histImage );

		//InPort̉f̎擾
		memcpy(inputImage->imageData,(void *)&(m_orig_img.pixels[0]),m_orig_img.pixels.length());

		//	Lv`ꂽ摜resultImageɃRs[AHSV\FnɕϊhsvImageɊi[
		cvCopy( inputImage, resultImage, NULL );
		cvCvtColor( resultImage, hsvImage, CV_BGR2HSV );

		//WindowEvent̎擾ɑ΂鏈
		if(m_eventIn.isNew() && m_xIn.isNew() && m_yIn.isNew()){
			m_xIn.read();
			m_yIn.read();
			m_eventIn.read();

			x = m_x.data;
			y = m_y.data;
			mouse_event = m_event.data;

			on_mouse(mouse_event, x, y, 0, 0);

			x= 0;
			y= 0;
			mouse_event = 0;
		}
		
		//	trackObjecttOTRACKING_STOPȊOȂAȉ̏s
        if( trackObject != TRACKING_STOP ){
			
			//ǐ՗̈̃qXgOvZƕ`
			CalculateHist(	hist, hsvImage, maskImage, selection );

			//	obNvWFNVvZ
            cvCalcBackProject( &hueImage, backprojectImage, hist );
            //	backProjection̂A}XN1łƂꂽ̂ݎc
			cvAnd( backprojectImage, maskImage, backprojectImage, 0 );

			//	CamShift@ɂ̈ǐՂs
			cvCamShift( backprojectImage, 
						trackWindow, 
						cvTermCriteria( CV_TERMCRIT_EPS | CV_TERMCRIT_ITER, 10, 1 ), 
						&trackComp, 
						&trackRegion );
			
			trackWindow = trackComp.rect;

			//	SnakeImagep̃O[XP[摜쐬
			cvCvtColor( resultImage, grayImage, CV_BGR2GRAY );

			if( backprojectMode == SHOW_BACKPROJECTION ){
                cvCvtColor( backprojectImage, resultImage, CV_GRAY2BGR );
			}
			if( resultImage->origin == 1 ){
                trackRegion.angle = -trackRegion.angle;
			}

			//	CamShiftł̗̈ǐՌʂSnakȅʒuɐݒ肷
			for( i=0; i<SEGMENT; i++ ){
				pt[i].x = cvRound(	trackRegion.size.width 
									* cos(i * 6.28 / SEGMENT + trackRegion.angle) 
									/ 2.0 + trackRegion.center.x );
				pt[i].y = cvRound(	trackRegion.size.height 
									* sin(i * 6.28 / SEGMENT + trackRegion.angle) 
									/ 2.0 + trackRegion.center.y );
			}
			//	Snakeɂ֊sos
			for( i=0; i<ITERATION_SNAKE; i++ ){
				cvSnakeImage(	grayImage, 
								pt, 
								SEGMENT, 
								&alpha, 
								&beta, 
								&gamma, 
								CV_VALUE, 
								window, 
								crit, 
								1);
				//	e֊s_̊ԂɐЂė֊s`悷
				for( j=0; j<SEGMENT; j++ ){
					if( j < SEGMENT-1 ){
						cvLine( resultImage, pt[j], pt[j+1], 
						  cvScalar(0,0,255,0), 2, 8, 0 );
					}
					else{ 
						cvLine( resultImage, pt[j], pt[0], 
						  cvScalar(0,0,255,0),  2, 8, 0 );
					}
				}
			}
        }

		//	}EXőI𒆂̏ǐ՗̈̐F𔽓]
		if( selectObject == SELECT_ON && selection.width > 0 && selection.height > 0 ){
			
			cvSetImageROI( resultImage, selection );
			cvXorS( resultImage, cvScalarAll(255), resultImage, 0 );
			cvResetImageROI( resultImage );
		}
		//	backprojectImage̍W_̏ꍇA㉺𔽓]
		if( backprojectImage->origin == 0 ){
			cvFlip( backprojectImage, backprojectImage, 0 );
		}
				
		//	摜\
		//cvShowImage( windowNameObjectTracking, resultImage );
		//key = cvWaitKey(1);

		// 摜f[^̃TCY擾
		double len = (resultImage->nChannels * resultImage->width * resultImage->height);
		double leng = (histImage->nChannels * histImage->width * histImage->height);
		m_out_img.pixels.length(len);
		m_hist_img.pixels.length(leng);

		// ỸC[WMemCopy
		memcpy((void *)&(m_out_img.pixels[0]), resultImage->imageData, len);
		memcpy((void *)&(m_hist_img.pixels[0]), histImage->imageData, leng);

		// ]摜f[^OutPorto͂B
		m_out_img.width = inputImage->width;
		m_out_img.height = inputImage->height;

		m_hist_img.width = inputImage->width;
		m_hist_img.height = inputImage->height;

		m_out_imgOut.write();
		m_hist_imgOut.write();
		
		if(inputImage != NULL){
			cvReleaseImage(&inputImage);
		}
		if(resultImage != NULL){
			cvReleaseImage(&resultImage);
		}
		if(hsvImage != NULL){
			cvReleaseImage(&hsvImage);
		}
		if(hueImage != NULL){
			cvReleaseImage(&hueImage);
		}
		if(hueImage != NULL){
			cvReleaseImage(&maskImage);
		}
		if(backprojectImage != NULL){
			cvReleaseImage(&backprojectImage);
		}
		if(grayImage != NULL){
			cvReleaseImage(&grayImage);
		}
		if(histImage != NULL){
			cvReleaseImage(&histImage);
		}
	
	}

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ObjectTracking::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectTracking::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectTracking::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectTracking::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectTracking::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ObjectTrackingInit(RTC::Manager* manager)
  {
    coil::Properties profile(objecttracking_spec);
    manager->registerFactory(profile,
                             RTC::Create<ObjectTracking>,
                             RTC::Delete<ObjectTracking>);
  }
  
};


